import { DisplayValue } from './display-value.js';
import { randomID } from '../../util/helpers.js';
import { Rectangle, Vector } from '../supporting/geometry/index.js';

export class Box {
  constructor(name, parentEl, options = {}) {
    this.name = name;
    const { tagName = 'div' } = options;
    this.el = document.createElement(tagName);
    this.el.box = this;
    const id = options.id ?? randomID();
    this.el.id = `${parentEl.id}_box_${id}`;
    this.el.classList.add('box');
    if (name) {
      this.el.setAttribute('box-name', name);
    }
    if (parentEl) {
      if (options.prepend) {
        parentEl.prepend(this.el);
      } else {
        parentEl.appendChild(this.el);
      }
    }
    this.position = options.position ?? Vector.zeros(2);
  }

  flex({ center = false } = {}) {
    this.addClass('flex');
    if (center) {
      this.addClass('flex-center');
    }
    return this;
  }

  monospace() {
    this.addClass('monospace');
    return this;
  }

  addClass(className) {
    this.el.classList.add(className);
    return this;
  }

  addBox(name) {
    const box = new Box(name, this.el);
    return box;
  }

  addDisplayValue(value) {
    const box = this.addBox(value.name).flex();
    return new DisplayValue(value, box);
  }

  setInnerHTML(html) {
    this.el.innerHTML = html;
    return this;
  }

  getId() {
    return this.el.id;
  }

  get rect() {
    const {
      width, height,
    } = this.el.getBoundingClientRect();
    return new Rectangle(this.position, [width, height]);
  }

  move(vector) {
    this.position = this.position.add(vector);
    this.el.style.left = `${Math.floor(this.position[0])}px`;
    this.el.style.top = `${Math.floor(this.position[1])}px`;
    // this.el.dispatchEvent(new CustomEvent('moved'));
  }
}
