import {
  DEFAULT_OVERLAP_FORCE,
  DEFAULT_TARGET_RADIUS,
  DISTANCE_FACTOR,
} from '../../util/constants.js';
import { Rectangle, Vector } from '../supporting/geometry/index.js';

const getRectangles = (boxes) => boxes.map((box) => (box instanceof Rectangle ? box : box.rect));
const getCenters = (boxes) => getRectangles(boxes).map((rect) => rect.center);

export const overlapRepulsionForce = (boxA, boxB, force = DEFAULT_OVERLAP_FORCE) => {
  const [rectA, rectB] = getRectangles([boxA, boxB]);
  const [centerA, centerB] = getCenters([rectA, rectB]);
  const r = centerB.subtract(centerA);

  // Apply a stronger force when overlap occurs
  if (!rectA.doesOverlap(rectB)) {
    return Vector.zeros(rectA.dim);
  }
  // If their centers actually coincide we can just randomize the direction.
  if (r.magnitudeSquared === 0) {
    return Vector.randomUnitVector(rectA.dim).scale(force);
  }
  return r.normalize().scale(force);
};

export const targetRadiusForce = (boxA, boxB, targetRadius = DEFAULT_TARGET_RADIUS) => {
  const [centerA, centerB] = getCenters([boxA, boxB]);
  const r = centerB.subtract(centerA);

  // Repel if closer than targetRadius
  // Attract if farther than targetRadius
  const force = -DISTANCE_FACTOR * (r.magnitude - targetRadius);
  return r.normalize().scale(force);
};
