export class Vector extends Array {
  get dim() {
    return this.length ?? 0;
  }

  add(vector) {
    if (vector.dim !== this.dim) {
      throw new Error('Can only add vectors of the same dimensions');
    }
    return Vector.from(this.map((q, idx) => q + vector[idx]));
  }

  subtract(vector) {
    if (vector.dim !== this.dim) {
      throw new Error('Can only subtract vectors of the same dimensions');
    }
    return Vector.from(this.map((q, idx) => q - vector[idx]));
  }

  static unitVector(dim, totalDim) {
    return Vector.from(Array(totalDim), (_, idx) => (idx === dim ? 1 : 0));
  }

  get magnitudeSquared() {
    return this.reduce((total, q) => total += q ** 2, 0);
  }

  get magnitude() {
    return Math.sqrt(this.magnitudeSquared);
  }

  scale(factor) {
    return Vector.from(this.map((q) => q * factor));
  }

  normalize() {
    return this.scale(1 / this.magnitude);
  }

  static randomUnitVector(totalDim) {
    return Vector.from(Array(totalDim), () => Math.random()).normalize();
  }

  static zeros(totalDim) {
    return Vector.from(Array(totalDim), () => 0);
  }
}
