import { DEFAULT_REP_TOKEN_TYPE_ID } from '../../../util/constants.js';
import { mochaRun } from '../../../util/helpers.js';
import { ForumTest } from '../forum.test-util.js';

describe('Forum', function tests() {
  this.timeout(0);

  const forumTest = new ForumTest({ displayAuthors: true });

  before(async () => {
    await forumTest.setup();
    await forumTest.newExpert();
    await forumTest.newExpert();
  });

  context('Multiple posts with overlapping authors', async () => {
    let forum;
    let experts;
    let posts;
    let dao;

    before(async () => {
      forum = forumTest.forum;
      experts = forumTest.experts;
      posts = forumTest.posts;
      dao = forumTest.dao;
    });

    it('Post1 with two authors', async () => {
      const authors = [
        { author: experts[0], weight: 0.5 },
        { author: experts[1], weight: 0.5 },
      ];
      await forumTest.addPost(authors, 10);
      forum.getPost(posts[0]).value.should.equal(10);

      dao.reputation.valueOwnedBy(experts[0].reputationPublicKey, DEFAULT_REP_TOKEN_TYPE_ID).should.equal(5);
      dao.reputation.valueOwnedBy(experts[1].reputationPublicKey, DEFAULT_REP_TOKEN_TYPE_ID).should.equal(5);
      dao.reputation.valueOwnedBy(experts[2].reputationPublicKey, DEFAULT_REP_TOKEN_TYPE_ID).should.equal(0);
    });

    it('Post2 with two authors, one shared with Post1', async () => {
      const authors = [
        { author: experts[1], weight: 0.5 },
        { author: experts[2], weight: 0.5 },
      ];
      await forumTest.addPost(authors, 10);
      forum.getPost(posts[0]).value.should.equal(10);

      dao.reputation.valueOwnedBy(experts[0].reputationPublicKey, DEFAULT_REP_TOKEN_TYPE_ID).should.equal(5);
      dao.reputation.valueOwnedBy(experts[1].reputationPublicKey, DEFAULT_REP_TOKEN_TYPE_ID).should.equal(10);
      dao.reputation.valueOwnedBy(experts[2].reputationPublicKey, DEFAULT_REP_TOKEN_TYPE_ID).should.equal(5);
    });
  });
});

mochaRun();
